/* $Workfile: OtxCst.h $   $Revision: 6 $    $Modtime: 10/14/05 6:49p $  */
#ifndef OTX_CST_H
#define OTX_CST_H
/*******************************************************************************

  @doc PublicHwApi OtxCst

  Copyright (C) 2004 - 2005 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxCst | OTX Logical Device Function Prototypes for C54CST DSP related devices

  @contents2
  OtxCst.h |

  @index func | OtxCst

  @contents2
  OTX C54CST DSP Related Functions - OtxCstXXX() | Functions available for logical devices hosted by the C54CST DSP

  @head3 OtxCstXXX() Function Reference |

  @index func | OtxCst

*******************************************************************************/

#include "OtxType.h"
#include "OtxDef.h"

#include "OtxCstD.h"
#include "OtxDataD.h"


#ifdef __cplusplus
extern "C" {
#endif



/*******************************************************************************

  @func
    Initialize the DTMF sequence generator. The time between tone can be 
    specified in the 'nSilence' parameter, and the duration of each tone 
    is determined by the valued of the 'nDuration' parameter.

  @comm
    To set the number to be dialled, call function <f OtxCstDtmfDial>

    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create DTMF logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_DTMF_DIALER>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstDtmfDialerInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat       // @parm Data Format of the Generated DTMF digits
);


/*******************************************************************************

  @func
    Set the number to be dialled by the DTMF dialler.

  @comm
    Dialling is started immediately when this function is called.

  @ex |

    char *g_sqNumber0 = "0123456789ABCD*#";
    OTX_INT32 nDigits = strlen(g_sqNumber0);
    OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialer[nCurrentLi];

    // sending digits on current Line, will receive Event when completed
    result = OtxCstDtmfDial(
                hHandle, 
                (OTX_UINT8 *)g_sqNumber0, 
                nDigits, 
                DEMO_TASK_REF, 
                OTX_TIME_INFINITY); 

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_DTMF_DIALER>
    <nl><f OtxCstDtmfDialerInit>
    <nl><f OtxCstPotsPulseDial>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstDtmfDial(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OTX_UINT8      sqDigits[],       // @parm Digits to send
  IN OTX_INT32      nDigits,          // @parm Number of digits to send
  IN OTX_TASK_REF   nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME       nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Configures the Voice Activity Detector (VAD). 


  @ex |

      // create silence Detector

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><t OtxDataFormatE>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_VOICE_ACTIVITY_DETECTOR>
    <nl><f OtxCstVoiceActivityStatus>
    <nl><f OtxCstDtmfDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstVoiceActivityDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat       // @parm Data Format of the Monitored Data Stream
);


/*******************************************************************************

  @func
    Reports whether the silence detector has detected silence or a tone

  @ex |

    OTX_BOOL  bSilence;      

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_VOICE_ACTIVITY_DETECTOR>
    <nl><f OtxCstVoiceActivityDetectorInit>
    <nl><f OtxCstDtmfDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstVoiceActivityDetectorStatus(
  IN  OTX_HANDLE    hDevice,                               // @parm Handle to the Logical device
  OUT OtxCstVoiceActivityEventCodesE  *eCurrentVadStatus   // @parm Current voice activity status. See <e OtxCstVoiceActivityEventCodesE.OTX_CST_VOICE_ACTIVITY_DETECTOR_EC_VOICE>
);


/*******************************************************************************

  @func
    Configures the DTMF Detector.

  @comm
    The received digits can either be stored in the Event Queue and be
    extracted with the appropriate Read function, or they can be stored 
    internally in which case only the most recently received digit can be 
    retrieved with a call to function <f OtxCstDtmfDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  see example on <f OtxCstDtmfDialerInit>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_DTMF_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxCstDtmfDetectorStatus>
    <nl><f OtxCstVoiceActivityDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstDtmfDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_BOOL       bStoreInQueue     // @parm If OTX_TRUE, generate events and store the received digits
                                      // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                      // with function <f OtxCstDtmfDetectorStatus>
);

/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received DTMF tone. If a tone was received, the function returns OTX_S_OK 
    and the last digit is in the cDtmfDigit parameter. If no tone was detected, 
    the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_DTMF_DETECTOR>
    <nl><f OtxCstDtmfDetectorInit>
    <nl><f OtxCstVoiceActivityDetectorStatus>

  @todo

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstDtmfDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT char          *cDtmfDigit       // @parm Most recent DTMF digit detected (or \0 if none)
);


/*******************************************************************************

  @func
    Initialize the Comfort Noise Generator. 

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create Comfort Noise Generator logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstComfortNoiseGeneratorInit(
  IN OTX_HANDLE hDevice,
  IN OtxDataFormatE eDataFormat      // @parm Data Format of input and output
);


/*******************************************************************************

  @func
    Basic initialization of the Voice Mixer. The mixer can take multiple input pipes and mix 
    them to one output pipe.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create Voice Mixer logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstVoiceMixerInit(
  IN OTX_HANDLE hDevice,
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of input and output
  IN OTX_INT32 nInputs                // @parm Number of inputs
);


/*******************************************************************************

  @func
    Extended initialization the Voice Mixer. The mixer can take multiple input pipes and mix 
    them to one output pipe. This function takes additional parameters such as separate formats
    for each input and the output. The digital gain for inputs and the output is also passed
    to the device by parameters.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create Voice Mixer logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstVoiceMixerInitEx(
  IN OTX_HANDLE hDevice,
  IN OtxDataFormatE sqInDataFormats[],  // @parm Data Format of each individual input
  IN OtxDataFormatE eOutDataFormat,     // @parm Data Format of the output
  IN OTX_INT16 sqInDigGains[],          // @parm Digital gain of each individual input (-1=-3dB, 0=0dB, 1=+3dB, etc)
  IN OTX_INT16 nOutDigGain,             // @parm Digital gain of the output (-1=-3dB, 0=0dB, 1=+3dB, etc)
  IN OTX_INT32 nInputs                  // @parm Number of inputs
);


/*******************************************************************************

  @func
    Initialize the (G.165/G.168) Voice Echo Canceller. 

    The Echo Canceller is used for cancellation of echo created by telephone hybrid,
    and conforms to G.165 and G.168 ITU recommendations. The user can set
    the value of maximum echo path equal to 16, 32, or 64 ms. Input/output signal
    range should be within standard PCM samples range.
    
    The echo canceller takes two input pipes, Once input pipe is the Near End (Ne) 
    has the echo 
    signal, and one input pipe is reference signal (Fe). The device will cancel 
    the reference signal from the signal with the echo (Ne). The output is one 
    pipe without the echo.

  @comm
    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create Echo Cancel logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstEchoCancelInit(
  IN OTX_HANDLE hDevice,
  IN OtxDataFormatE eDataFormat,      
  IN OtxCstEchoCancelMaxEchoPathE eMaxEchoPath
);


/*******************************************************************************

  @func
    Configures the Call Progress Tone (CPT) Detector.

  @comm
    The detected tone or tone series (symbols) can either be stored in the Event 
    Queue and be extracted with the appropriate Read function, or they can be stored 
    internally in which case only the most recently received symbol can be 
    retrieved with a call to function <f OtxCstCptDetectorStatus>

    Start detection by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

    see example for <f OtxCstCptDialerInit>

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_CPT_DETECTOR>
    <nl><t OtxDataFormatE>
    <nl><f OtxCstCstDetectorStatus>
    <nl><f OtxCstVoiceActivityDetectorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstCptDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Monitored Data Stream
  IN OTX_BOOL       bStoreInQueue,    // @parm If OTX_TRUE, generate events and store the received digits
                                      // in the Event QueusFifo, If OTX_FALSE Don't store. Provide polling test
                                      // with function <f OtxCstDtmfDetectorStatus>
  IN OtxCstCptDetectorConfigurationE eCptConfiguration // @parm The CPT device can be configured to detect different CPT tones (for differerent geographical regions). See <t OtxCstCptDetectorConfigurationE>.
);


/*******************************************************************************

  @func
    This function allows the user to poll and check for the most recently
    received Call Progress Tone symbol. If a tone was received, the function 
    returns OTX_S_OK and the symbol is in the <p nSymbol> parameter. If no 
    tone has been detected, the function returns OTX_S_FALSE.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_CPT_DETECTOR>
    <nl><f OtxCstCptDetectorInit>
    <nl><f OtxCstVoiceActivityDetectorStatus>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstCptDetectorStatus(
  IN  OTX_HANDLE    hDevice,          // @parm Handle to the Logical device
  OUT OTX_INT32    *nSymbol           // @parm Most recent CPT symbol detected (or -1 if none)
);


/*******************************************************************************

  @func
    Initialize the CPT generator.

  @comm
    To dial a certain CPT tone call function <f OtxCstCptGenerateTone>

    Start generation by calling the function <f OtxDrvEnable> and stop
    generation by calling the function <f OtxDrvDisable>

  @ex |

    // create CPT logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_CPT_GENERATOR>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstCptGeneratorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the Generated CPT digits
  IN OtxCstCptGeneratorConfigurationE eCptConfiguration  // @parm The CPT device can be configured to detect different CPT tones (for differerent geographical regions). See <t OtxCstCptGeneratorConfigurationE>.
);


/*******************************************************************************

  @func
    Start generation of a specified Call Progress Tone.

  @comm
    The tone is started immediately when this function is called if the device
    is enabled.

  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_CPT_GENERATOR>
    <nl><f OtxCstCptGeneratorInit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstCptGenerateTone(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxCstCptGeneratorToneE eTone    // @parm Call Progress Tone to be generated
);


/*******************************************************************************

  @func
    Initialize the POTS Line Interface logical device hosted by the C54CST. 
    This device can be configured for various geographical areas. For a list
    the supported areas please refer to <t OtxCstPotsCfgSetE>.


  @ex |

    // create CST POTS LI logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_LI_POTS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of data from the POTS device
  IN OtxCstPotsCfgSetE eCfgSet        // @parm Geographical area to configure this POTS Line interface device for. Please refer to <t OtxCstPotsCfgSetE>.
);


/*******************************************************************************

  @func
    Make the POTS line device to go off-hook

  @comm
    This function does not wait for a dial tone, to wait for a dial tone, 
    an Audio Receiver must be associated with the Line Device. This is done 
    by installing the Audio module and connecting PotsLine timeslots to 
    Audio Receivers in the Audio Module by means of the Tss module.

    No operation if already off-hook.

  @ex |

    hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];

    // set active POTS Line Off-Hook
    result = OtxCstPotsOffHook(hCurrentPotsLine); 

  @xref
    <f OtxCstPotsOnHook>
    <nl><f OtxCstPotsHookFlash>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsOffHook(
  IN OTX_HANDLE       hDevice,          // @parm A handle to a POTS line device.
  IN OTX_TASK_REF     nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Make the POTS line device go on-hook

  @comm
    No operation if already off-hook.

  @ex |
  
    hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
    
    // set active POTS Line On-Hook
    result = OtxCstPotsOnHook(hCurrentPotsLine); 

  @xref
    <f OtxCstPotsOffHook>
    <nl><f OtxCstPotsHookFlash>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsOnHook(
  IN OTX_HANDLE       hDevice,          // @parm A handle to a POTS line device.
  IN OTX_TASK_REF     nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Make the POTS line device go on-hook in line monitor mode

  @comm
    No operation if already off-hook.

  @ex |
  
    hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
    
    // set active POTS Line On-Hook
    result = OtxCstPotsOnHookLineMon(hCurrentPotsLine); 

  @xref
    <f OtxCstPotsOffHook>
    <nl><f OtxCstPotsHookFlash>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsOnHookLineMon(
  IN OTX_HANDLE       hDevice,          // @parm A handle to a POTS line device.
  IN OTX_TASK_REF     nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    An atomic on-hook off-hook operation

  @comm
    A non-operation if the device is not off-hook. This function is a
    blocking call and will not return until the process is completed.

  @ex |
    
    hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];

    // flash Hook on current POTS line
    result = OtxCstPotsHookFlash(
                hCurrentPotsLine, 
                200   // Stay on-hook for 200 ms
                ); 

  @xref
    <f OtxCstPotsOffHook>
    <nl><f OtxCstPotsOnHook>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsHookFlash(
  IN OTX_HANDLE       hDevice,            // @parm A handle to a POTS line device.
  IN OTX_TIME         nDurationMs,        // @parm Time to stay on-hook.
  IN OTX_TASK_REF     nTaskRef,           // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs          // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Dial a digit in the form of legacy pulse groups.

  @comm
    A pulse group is realized as group of pulses. Example, in Sweden a
    '0' is one pulse, and '1' are two pulses.

    A pulse is a disconnection of the line, and it is therefore a
    prerequisite that the line is off-hook prior to calling this
    function.

  @ex |
    

  @xref
    <f OtxCstDtmfDial>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsPulseDial(
  IN OTX_HANDLE       hDevice,            // @parm A handle to a POTS line device.
  IN OTX_INT32        nDigit,             // @parm Digit to be dialled (0-9)
  IN OTX_TASK_REF     nTaskRef,           // @parm Task reference, see also <t OTX_TASK_REF>.
  IN OTX_TIME         nWaitMaxMs          // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Read the Line voltage, loop current, or on-hook line monitor. The value
    that is read depends on the state of the Pots Li (off-hook, on-hook, or 
    on-hook-line-monitor).

    If the read operation is succesful, a <e OtxCstPotsEventCodeE.OTX_CST_POTS_SENSE_LINE_COMPLETE>
    event is generated.

  @xref
    <e OtxCstPotsEventCodeE.OTX_CST_POTS_SENSE_LINE_COMPLETE>
    <nl><f OtxCstPotsOffHook>
    <nl><f OtxCstPotsOnHook>
    <nl><f OtxCstPotsOnHookLineMon>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstPotsSenseLine(
  IN OTX_HANDLE       hDevice,          // @parm A handle to a POTS line device.
  IN OTX_TASK_REF     nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Initialize the Caller ID (CID) detector.

  @comm
    Start detection by calling the function <f OtxDrvEnable> and stop
    detection by calling the function <f OtxDrvDisable>

  @ex |

    // create CID logical devices


  @xref
    <f OtxDrvCreateLogicalDevice>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_CST_CALLER_ID_DETECTOR>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxCstCidDetectorInit(
  IN OTX_HANDLE     hDevice,          // @parm Handle to the Logical device
  IN OtxDataFormatE eDataFormat,      // @parm Data Format of the samples that is delivered to this device 
  IN OtxCstCidDetectorConfigurationE eCpidConfiguration  // @parm The CID device can be configured to detect different CPT tones (for differerent geographical regions). See <t OtxCstCidGeneratorConfigurationE>.
);



#ifdef __cplusplus
}  // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2004 - 2005


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif  // OTX_CST_H
